# -*- coding: utf-8 -*-
"""
基于gensim模块的中文句子相似度计算

思路如下：
1. 分词
2. 去停用词
3. 关键词提取
"""

import jieba
import jieba.analyse
import os


class ExtractKeywords(object):
    """

    :param ori_text:
    :return:
    """

    def __init__(self, ori_text=None, _stopwords_path=None, cut_all='search', _user_dict_path=None):
        """
        初始化

        - _stopwords_path: 停用词词表
        - _user_dict_path: 自定义词表
        - cut_all: 默认切词方式, ['search', True, False] -> [浏览器切词, 全切, 简切]
        """
        self.ori_text = ori_text
        self.cut_all = cut_all

        current_dir_path = os.path.dirname(__file__)
        self._stopwords_path = os.path.join(current_dir_path,
                                            'stopwords.txt') if _stopwords_path is None else _stopwords_path

        self._user_dict_path = os.path.join(current_dir_path,
                                            'user_dict.txt') if _user_dict_path is None else _user_dict_path

    @staticmethod
    def get_stop_word(_stopwords_path):
        """
        获取停用词
        :stopwords_path:
        :return:
        """
        stopwords = set()
        file = open(_stopwords_path, 'r', encoding='UTF-8')
        for line in file:
            stopwords.add(line.strip())
        file.close()
        return stopwords

    _loaded_user_dc = False

    _stopwords_set = None

    @property
    def stopwords_set(self):
        if not self._stopwords_set:
            self._stopwords_set = self.get_stop_word(self._stopwords_path)
        return self._stopwords_set

    _user_dict = None

    @property
    def user_dict(self):
        if not self._user_dict:
            self._user_dict = self.get_stop_word(self._user_dict_path)
        return self._user_dict

    def handle(self, ori_text=None, cut_all=None, topK=10):
        if not self._loaded_user_dc:       # 是否已加载自定义词典
            jieba.load_userdict(self._user_dict_path)
            self._loaded_user_dc = True

        # step1: break down words
        # 精确模式
        # print(self.ori_text)
        if ori_text is None:
            ori_text = self.ori_text

        if cut_all is None:
            cut_all = self.cut_all

        if cut_all == 'search':
            cut_text = jieba.cut_for_search(ori_text)
        else:
            cut_text = jieba.cut(ori_text, cut_all=cut_all)

        # step2：去停用词
        # 这里是有一个文件存放要改的文章，一个文件存放停用表，然后和停用表里的词比较，一样的就删掉，最后把结果存放在一个文件中
        # stopwords_set = self.get_stop_word(self._stopwords_path)

        final = ""
        for word in cut_text:
            if word not in self.stopwords_set:
                final = final + " " + word
        # print('--- 分词结果: ', final)

        # step3：提取关键词
        # res = jieba.analyse.extract_tags(final, topK=5, withWeight=True, allowPOS=())
        # n	普通名词	f	方位名词	s	处所名词	t	时间
        # nr	人名	ns	地名	nt	机构名	nw	作品名
        # nz	其他专名	v	普通动词	vd	动副词	vn	名动词
        # a	形容词	ad	副形词	an	名形词	d	副词
        # m	数量词	q	量词	r	代词	p	介词
        # c	连词	u	助词	xc	其他虚词	w	标点符号
        # PER	人名	LOC	地名	ORG	机构名	TIME	时间

        res = jieba.analyse.extract_tags(final, topK=topK,
                                         allowPOS=('n', 'f', 's', 'nr', 'ns', 'nt', 'nw', 'nz', 'v', 'vn', 'an',
                                                   'PER', 'LOC', 'ORG'))

        # res = jieba.analyse.extract_tags(final, topK=10)
        # res = final

        # text 为待提取的文本
        # topK:返回几个 TF/IDF 权重最大的关键词，默认值为20。
        # withWeight:是否一并返回关键词权重值，默认值为False。
        # allowPOS:仅包括指定词性的词，默认值为空，即不进行筛选。
        if res:
            return res
        else:
            return [self.ori_text]


extract_keywords = ExtractKeywords()

if __name__ == "__main__":
    ori_text = '城市绿化是栽种植物以改善城市环境的活动。 ' \
               '城市绿化作为城市生态系统中的还原组织 ' \
               '城市生态系统具有受到外来干扰和破坏而恢复原状的能力，' \
               '就是通常所说的城市生态系统的还原功能。'
    ori_text2 = '信息安全事故'
    # ori_text2 = '特大安全事故'
    # ori_text2 = '安全'
    # ori_text2 = '事故'
    # _stopwords_path = r'E:\AProject\EmergencyManagement_dj2201\common_app\common\stopwords.txt'
    # _stopwords_path = r'G:\PycharmProjects\yingJiGuanLiXiTong_dj2201\common_app\common\stopwords.txt'
    a = ExtractKeywords(ori_text2)
    res = a.handle()
    print(type(res), res)



import os
import shutil
import sys
import argparse
from bdtime import tt


dirname = 'shells'


def strtobool(val):
    """Convert a string representation of truth to true (1) or false (0).

    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    elif val in ('n', 'no', 'f', 'false', 'off', '0', 'none', 'null'):
        return 0
    else:
        raise ValueError("invalid truth value %r" % (val,))


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--name", type=str, default="bode-shells",
                        help="the name of this experiment")
    # parser.add_argument("--seed", type=int, default=1,
    #                     help="seed of the experiment")
    # parser.add_argument("--float", type=float, default=1.2345,
    #                     help="float")
    parser.add_argument(
        '-mf', "--move-file", type=lambda x: bool(strtobool(x)), default=False, nargs="?", const=True,
        help="if toggled, program will auto move files to current work directory like forceManage.py."
    )

    parser.add_argument(
        '-ow', "--overwrite", type=lambda x: bool(strtobool(x)), default=False, nargs="?", const=True,
        help="强制覆盖脚本文件夹."
    )

    # parser.add_argument('-ls', '-list', action='append')      # 输入列表

    args = parser.parse_args()
    return args


def run():
    args = parse_args()

    if args.overwrite:
        if os.path.exists(dirname):
            print(f'*** 将强制覆盖{dirname}!')
            tt.sleep(1)
            shutil.rmtree(dirname)
    assert not os.path.exists(dirname), f'脚本文件夹[{dirname}]已存在!'

    # from bdtime import show_json
    # show_json(args.__dict__)
    # return

    # 保存凭证
    cmd = 'git config credential.helper store'
    os.system(cmd)

    # clone shells
    cmd = 'git clone https://gitee.com/bode135/shells.git'
    os.system(cmd)

    if not args.move_file:
        return

    print(f'--- 移动部分脚本文件: {args.move_file}')
    filename = 'forceManage.py'
    src = os.path.join(dirname, filename)
    dst = filename
    shutil.move(src, dst)       # shutil.copy2(src, dst)


if __name__ == '__main__':
    print('sys.argv --- ', sys.argv)
    run()



from django.db import models
from django.db.models import F
from django.http import JsonResponse
from django.contrib.postgres.search import SearchVectorField
from django.contrib.postgres.indexes import GinIndex
from django.contrib.postgres.search import SearchVector, SearchQuery, SearchRank
from django.contrib import admin
from .extract_keyword import extract_keywords as _extract_keywords


# region # --- 默认检索权重设置

"""
# --- 若WEIGHT_CONF不为None, 则默认值为0, 可实现单字段检索.

# Django默认用[D, C, B, A]代表该字段的权重
WEIGHT_CONF = {
    'D': 0.2,
    'C': 0.4,
    'B': 0.6,
    'A': 0.8,
}
"""

WEIGHT_CONF = None

RANK__GTE = 0.001        # 相关性检索过滤的最小值

# endregion


# region # --- 不需要改动的变量

search_conf_name = 'search_field_conf'        # 检索配置的字段名
search_vector_field_name = 'search_vector'      # 检索字段名
search_weight_name = 'search_weight_conf'        # 检索权重配置名
search_rank__gte = 'search_rank__gte'       # 关性检索过滤的最小值


_WEIGHTS_ORDERING = ['D', 'C', 'B', 'A']       # Django权重的默认顺序, 不能乱改!

# endregion


def get_search_vector(search_conf, config="chinese_zh"):
    vector = None
    for k, v in search_conf.items():
        if vector:
            vector += SearchVector(k, weight=v, config=config)
        else:
            vector = SearchVector(k, weight=v, config=config)
    return vector


def extract_keywords(keywords, cut_all=None):
    keywords = _extract_keywords.handle(keywords, cut_all=cut_all)
    return keywords


class SearchManager(models.Manager):
    _weight_conf = None
    _rank__gte = None

    @property
    def weight_conf(self):
        if self._weight_conf is None:
            if hasattr(self.model, self.search_weight_name) and getattr(self.model, self.search_weight_name):
                self._weight_conf = getattr(self.model, self.search_weight_name)
            else:
                global WEIGHT_CONF
                self._weight_conf = WEIGHT_CONF
        return self._weight_conf

    @property
    def rank__gte(self):
        if self._rank__gte is None:
            if hasattr(self.model, self.search_rank__gte) and getattr(self.model, self.search_rank__gte):
                self._rank__gte = getattr(self.model, self.search_rank__gte)
            else:
                global RANK__GTE
                self._rank__gte = RANK__GTE
        return self._rank__gte

    @property
    def search_rank__gte(self):
        global search_rank__gte
        return search_rank__gte

    @property
    def search_conf_name(self):
        global search_conf_name
        return search_conf_name

    @property
    def search_weight_name(self):
        global search_weight_name
        return search_weight_name

    @property
    def weights_ordering(self):
        global _WEIGHTS_ORDERING
        return _WEIGHTS_ORDERING

    def with_documents(self):
        assert hasattr(self.model, self.search_conf_name), f'模型{self.model}没有检索字段`{self.search_conf_name}`'
        search_conf = getattr(self.model, self.search_conf_name)
        vector = get_search_vector(search_conf)
        ret = self.get_queryset().annotate(_document=vector)
        return ret

    def search(self, keywords, queryset=None, weight_conf=None, cut_all=None, auto_order=True, rank__gte=None):
        """
        综合检索

        :param keywords: 检索关键词
        :param queryset: 基于该queryset进行检索, 默认全部
        :param weight_conf: 权重配置, 为空则使用模型的weight_conf, 模型的weight_conf为空则使用默认的`WEIGHT_CONF`
        :param cut_all: jieba的切词类型, 默认`search`. 取值: {"search": 浏览器检索, True: 全切, False: 简切}.
        :param auto_order: 自动按相关性大小(rank值)排序
        :param rank__gte: 最小相关性, 为空则使用模型的对应属性, 模型对应属性为空则使用`RANK__GTE`
        :return: 检索结果的`queryset`
        """
        keywords = extract_keywords(keywords, cut_all=cut_all)
        # print(f"*** search ExtractKeywords: {keywords} --- cut_all: {cut_all}")

        query = None
        for kw in keywords:
            tmp = SearchQuery(kw)
            query = tmp if query is None else (query | tmp)

        queryset = queryset if queryset is not None else self.get_queryset()

        # 实现可以自定义权重的检索(包括前端层面自定义和各个模型层面自定义)
        _weight_conf = weight_conf if weight_conf is not None else self.weight_conf
        if _weight_conf is None:
            queryset = queryset.annotate(rank=SearchRank(F(search_vector_field_name), query))
        else:
            weights = [_weight_conf.get(w, 0) for w in self.weights_ordering]
            queryset = queryset.annotate(rank=SearchRank(F(search_vector_field_name), query, weights=weights))

        rank__gte = rank__gte if rank__gte is not None else self.rank__gte
        queryset = queryset.filter(**{search_vector_field_name: query}).filter(**{'rank__gte': rank__gte})
        if auto_order:
            queryset = queryset.order_by('-rank', 'id')

        # from bddjango import show_ls, show_json
        # show_json(_weight_conf)
        # show_ls(queryset.values('id', 'rank')[:5])
        return queryset


class BaseFullTextSearchModel(models.Model):
    """
    # 全文检索类

    - 用户需要定义检索配置`search_field_conf`.
        * ps: 这个字段修改后得重新migrate数据库, 并admin后台手动更新GIndex.
    - 可选
        - search_weight_conf: 自定义权重配置
        - search_rank__gte: 相关性过滤

    ---

    - eg:

    ```
    search_field_conf = {
        'title': 'A',
        'content': 'B',
    }

    search_weight_conf = {
        'A': 0.9,
        'B': 0.4,
    }

    search_rank__gte = 0.3
    ```
    """
    search_field_conf = None
    search_weight_conf = None
    search_rank__gte = 0.001

    search_vector = SearchVectorField(null=True, blank=True)
    objects = SearchManager()

    @staticmethod
    def extract_keywords(keywords, cut_all=None):
        return extract_keywords(keywords, cut_all=cut_all)

    @property
    def search_vector_field_name(self):
        global search_vector_field_name
        return search_vector_field_name

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        if 'update_fields' not in kwargs or search_vector_field_name not in kwargs['update_fields']:
            instance = self._meta.default_manager.with_documents().get(pk=self.pk)
            setattr(instance, search_vector_field_name, instance._document)
            instance.save(update_fields=[search_vector_field_name])

    class Meta:
        indexes = [GinIndex(fields=[search_vector_field_name])]
        abstract = True


# region # --- AdminMixin

def rebuild_index(md):
    """
    # admin中用来更新索引

    ## reference:
       - https://juejin.cn/post/6844903480893636616
       - https://cainiaojiaocheng.com/Django/docs/3.2.x/ref/contrib/postgres/search
    """
    print('开始建立索引!', md._meta, md._meta.model_name)
    assert hasattr(md, search_conf_name), f'{md}没有{search_conf_name}字段!'

    search_conf: dict = getattr(md, search_conf_name)
    search_vector = get_search_vector(search_conf)
    md.objects.update(search_vector=search_vector)


class BuildIndexAdminMixin:
    """
    更新索引AdminMixin
    """

    @admin.action(permissions=['change', 'add'])
    def rebuild_index(self, request, queryset=None, model=None):
        from bdtime import Time
        tt = Time()
        count = queryset.count()
        if count == 0:
            return JsonResponse(data={
                'status': 'error',
                'msg': f'没有数据!'
            })

        rebuild_index(self.model)

        msg = f'成功建立{count}条索引, 耗时: {tt.now(2)}秒.'
        self.message_user(request, msg)

        return JsonResponse(data={
            'status': 'success',
            'msg': f'处理成功! 耗时: {tt.now(2)}秒.'
        })

    rebuild_index.short_description = "更新索引"
    rebuild_index.icon = 'fa fa-spinner'
    rebuild_index.confirm = '确定更新全部索引？'
    rebuild_index.layer = {
        'title': '确定更新全部索引?',
        'tips': '该操作可能比较耗时, 请耐心等候...',
    }


def _get_select_options(select_key, md, none_value_text='全部'):
    assert select_key is not None, '`select_key`不能为空!'

    from bddjango import conv_queryset_to_dc_ls

    qsv_ls = md.objects.distinct(select_key).order_by(select_key).values(select_key)
    option_dc_ls = conv_queryset_to_dc_ls(qsv_ls)

    select_options = []

    if none_value_text:
        none_value_dc = {
            'key': none_value_text,
            'label': none_value_text,
        }
        select_options.append(none_value_dc)

    for option_dc_i in option_dc_ls:
        v = option_dc_i.get(select_key)
        dc = {
            'key': v,
            'label': v,
        }
        select_options.append(dc)

    return select_options


def _get_a_new_test_search_vector_mixin():

    class _TestSearchVectorMixin:
        default_search_keywords = None
        select_key = None
        _select_options = None

        _none_value_text = '全部'

        @property
        def select_options(self):
            if self._select_options is None:
                if self.select_key is None:
                    return None
                # assert self.select_key, '属性`select_key`不能为空!'
                self._select_options = _get_select_options(self.select_key, self.model, self._none_value_text)
            return self._select_options

        def test_search_vector(self, request, queryset):
            from bddjango import show_ls, show_json
            from bdtime import tt

            tt.__init__()
            query_dc = request.POST

            select_value = query_dc.get(self.select_key)
            if select_value == self._none_value_text:
                select_value = None
            keywords = query_dc.get('keywords', self.default_search_keywords)
            if not keywords:
                return JsonResponse(data={
                    'status': 'error',
                    'msg': '检索关键词`search_keywords`不能为空!'
                })

            kw = BaseFullTextSearchModel.extract_keywords(keywords)

            if select_value:
                qs_ls = self.model.objects.filter(**{self.select_key: select_value})
                qs_ls = self.model.objects.search(keywords=keywords, queryset=qs_ls)
            else:
                qs_ls = self.model.objects.search(keywords=keywords)

            # show_ls(qs_ls.values('id', 'fen_lei_ming', 'wen_ti', 'rank')[:5])
            print(f"--- admin ExtractKeywords: {kw}")

            self._ajax_return_qs_ls = qs_ls

            msg_0 = f"处理成功! <br>耗时: {tt.now(3)}秒,<br>返回结果数量: {qs_ls.count()}条.<br>"
            msg_1 = f"""关键词抽取结果: {kw}"""
            msg_all = msg_0 + msg_1
            self.message_user(request, msg_all)
            return JsonResponse(data={
                'status': 'success',
                'msg': msg_0.replace('<br>', ' ')
            })

        test_search_vector.short_description = '综合检索测试'
        test_search_vector.type = 'success'
        test_search_vector.layer = {
            'title': '综合检索',
            'tips': '您好！您可以输入想要咨询的问题，我来为您解答！',
            'params': [
                {
                    # 这里的type 对应el-input的原生input属性，默认为input
                    'type': 'input',
                    # key 对应post参数中的key
                    'key': 'keywords',
                    # 显示的文本
                    'label': '检索条件',
                    # 为空校验，默认为False
                    # 'require': True
                }
            ]
        }

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

            select_key_attr_name = 'select_key'
            if hasattr(self, select_key_attr_name) and getattr(self, select_key_attr_name):
                select_key = getattr(self, select_key_attr_name)
                # print(f'--- key: {select_key} --- model: {self.model}')
                dc = {
                    'type': 'select',
                    'key': select_key,
                    'label': '类型',
                    'width': '200px',
                    # size对应elementui的size，取值为：medium / small / mini
                    'size': 'small',
                    # value字段可以指定默认值
                    'value': self._none_value_text,
                    'options': self.select_options,
                }
                # from bddjango import show_ls, show_json
                # show_json(dc)
                self.test_search_vector.layer['params'].insert(0, dc)

    return _TestSearchVectorMixin


class TestSearchVectorMixin:
    """
    # 测试全文检索的mixin

    - 覆写了__init__方法, 必须放在AdminClass的前面!
    - 必须每个class都用一个全新的Mixin, 避免属性共用问题
    """
    @staticmethod
    def get_new():
        # 避免迁移出错
        from .db_utils import judge_db_is_migrating
        if judge_db_is_migrating():
            class _:
                pass
            return _

        return _get_a_new_test_search_vector_mixin()

# endregion




import setuptools
from pgZhSearch import version
from m2r import parse_from_file


# with open("README.md", "r", encoding='utf-8') as fh:
#     long_description = fh.read()
long_description = parse_from_file('README.md')     # .markdown必须转换为.rst, 否则有可能报错

setuptools.setup(
    name="pgZhSearch",
    version=version(),
    author="bode135",
    author_email='2248270222@qq.com',   # 作者邮箱
    description="django-postgresql-zh-full-text-search",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://gitee.com/bode135/pgZhSearch', # 主页链接
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['psycopg2-binary==2.9.1', 'jieba'],     # 依赖模块
)



from django.contrib.auth import get_user_model


def create_user_if_not_exist(username, passwd, superuser=False, extra_fields=None):
    """
    创建一个用户
    """
    User = get_user_model()
    user = User.objects.filter(username=username)
    if not user.exists():
        print(f'--- 初始创建用户: {username}')

        fields_dc = {}
        fields_dc.update({'username': username, "password": passwd})

        if isinstance(extra_fields, dict):
            fields_dc.update(extra_fields)

        if superuser:
            User.objects.create_superuser(**fields_dc)
        else:
            User.objects.create_user(**fields_dc)
        return True
    else:
        return False



import psycopg2
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
import sys


def judge_db_is_migrating():
    if 'makemigrations' in sys.argv or 'migrate' in sys.argv:
        return True
    else:
        return False


def get_cursor_by_pg_conf(pg_conf, db_name=None):
    assert 'postgresql' in pg_conf.get('ENGINE'), '本方案仅支持postgresql数据库!'

    db_name = db_name or pg_conf.get('NAME')  # 想新建的数据库名

    user = pg_conf.get('USER')
    pwd = pg_conf.get('PASSWORD')
    port = pg_conf.get('PORT')
    host = pg_conf.get('HOST')

    conn = psycopg2.connect(database=db_name, port=port, host=host, user=user, password=pwd)

    conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)

    cursor = conn.cursor()
    return cursor


def create_db_if_not_exist(pg_conf, debug=False):
    """
    如果不存在指定数据库, 则创建.
    若创建了则返回True, 否则False
    """
    db_name = pg_conf.get('NAME')

    cursor = get_cursor_by_pg_conf(pg_conf, db_name="postgres")

    # 检查数据库是否存在
    # sql = f'SELECT u.datname FROM pg_catalog.pg_database u where u.datname="{db_name}";'
    # db_name = "a2203_zhiHuiTuShuGuanPingTai_20220618"
    sql = f"SELECT u.datname FROM pg_catalog.pg_database u where u.datname='{db_name}';"
    cursor.execute(sql)

    row = cursor.fetchall()
    if row:
        msg = f'已存在数据库[{db_name}]!'
        if debug:
            print(msg)
        ret = False
    else:
        # 创建数据库
        sql = 'CREATE DATABASE "{}";'.format(db_name)     # pg不支持 CREATE DATABASE XXX IF NOT EXISTS;
        try:
            cursor.execute(sql)
            msg = f'数据库[{db_name}]创建成功.'
            print(msg)
            ret = True
        except Exception as e:
            # print(e)
            if 'already exists' in str(e):
                msg = f'已存在数据库[{db_name}]!'
            else:
                msg = f'--- create_db_if_not_exist Error: {e}'
            if debug:
                print(msg)
            ret = False

    cursor.close()
    return ret


def is_exist_table_in_db(pg_conf, table_name, debug=False):
    """
    判断db中是否已经迁移好了表table_name
    """
    cursor = get_cursor_by_pg_conf(pg_conf)

    # 判断表是否存在
    sql = f'SELECT count(*) FROM "{table_name}";'

    try:
        exist_flag = True
        cursor.execute(sql)
    except Exception as e:
        exist_flag = False
        if debug:
            print('*** is_exist_table_in_db error: ', e)

    if debug:
        print('--- is_exist_table_in_db:', exist_flag)

    cursor.close()
    return exist_flag


def use_zhparser_in_db(pg_conf):
    """
    启动指定pg库的中文分词解释器
    """
    cursor = get_cursor_by_pg_conf(pg_conf)

    try:
        sql ="CREATE EXTENSION zhparser;"
        cursor.execute(sql)
    except Exception as e:
        if 'already exists' in str(e):
            print('数据库已启用`zhparser`!')
            return False
        print('*** zhparser_1: ', e)

    try:
        sql = "CREATE TEXT SEARCH CONFIGURATION chinese_zh (PARSER = zhparser);"
        cursor.execute(sql)
    except Exception as e:
        print('*** zhparser_2: ', e)

    try:
        sql = "ALTER TEXT SEARCH CONFIGURATION chinese_zh ADD MAPPING FOR n,v,a,i,e,l WITH simple;"
        cursor.execute(sql)
    except Exception as e:
        print('*** zhparser_3: ', e)

    print('--- 成功启动zhparser解释器.')
    cursor.close()
    return True


def main():
    from ControlHub.settings.development import DATABASES
    pg_conf = DATABASES.get('default')

    create_db_if_not_exist(pg_conf, debug=True)

    use_zhparser_in_db(pg_conf)

    is_exist_table_in_db(pg_conf, table_name='user_customuser', debug=True)


if __name__ == '__main__':
    main()


